@@ -88,6 +88,7 @@ struct _VinoServerPrivate
   guint             use_upnp : 1;
   guint             disable_xdamage : 1;
   guint             notify_on_connect : 1;
+  guint             reject_incoming : 1;
 };
 
 struct _VinoClient
@@ -123,7 +124,8 @@ enum
   PROP_DISABLE_BACKGROUND,
   PROP_USE_UPNP,
   PROP_DISABLE_XDAMAGE,
-  PROP_NOTIFY_ON_CONNECT
+  PROP_NOTIFY_ON_CONNECT,
+  PROP_REJECT_INCOMING
 };
 
 static enum rfbNewClientAction vino_server_auth_client (VinoServer *server,
@@ -134,6 +136,8 @@ static enum rfbNewClientAction vino_server_auth_client (VinoServer *server,
 static void vino_server_setup_framebuffer     (VinoServer *server);
 static void vino_server_release_framebuffer   (VinoServer *server);
 static void vino_server_update_security_types (VinoServer *server);
+static void vino_server_set_notify_on_connect (VinoServer *server,
+    gboolean notify_on_connect);
 
 static gpointer parent_class;
 
@@ -502,6 +506,12 @@ vino_server_handle_new_client (rfbClientPtr rfb_client)
 
   g_return_val_if_fail (VINO_IS_SERVER (server), RFB_CLIENT_REFUSE);
 
+  if (server->priv->reject_incoming)
+    {
+      dprintf (RFB, "Reject client on fd %d\n", rfb_client->sock);
+      return RFB_CLIENT_REFUSE;
+    }
+
   dprintf (RFB, "New client on fd %d\n", rfb_client->sock);
 
   if (!server->priv->fb)
@@ -1196,6 +1206,12 @@ vino_server_set_property (GObject      *object,
     case PROP_DISABLE_XDAMAGE:
       vino_server_set_disable_xdamage (server, g_value_get_boolean (value));
       break;
+    case PROP_NOTIFY_ON_CONNECT:
+      vino_server_set_notify_on_connect (server, g_value_get_boolean (value));
+      break;
+    case PROP_REJECT_INCOMING:
+      vino_server_set_reject_incoming (server, g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1263,6 +1279,9 @@ vino_server_get_property (GObject    *object,
     case PROP_NOTIFY_ON_CONNECT:
       g_value_set_boolean (value, server->priv->notify_on_connect);
       break;
+    case PROP_REJECT_INCOMING:
+      g_value_set_boolean (value, server->priv->reject_incoming);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1505,7 +1524,15 @@ vino_server_class_init (VinoServerClass *klass)
                                                          G_PARAM_STATIC_NAME |
                                                          G_PARAM_STATIC_NICK |
                                                          G_PARAM_STATIC_BLURB));
-
+  g_object_class_install_property (gobject_class,
+                                   PROP_REJECT_INCOMING,
+                                   g_param_spec_boolean ("reject-incoming",
+                                                         "Reject incoming",
+                                                         "If TRUE reject incoming connections",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE   |
+                                                         G_PARAM_CONSTRUCT_ONLY |
+                                                         G_PARAM_STATIC_STRINGS));
 }
 
 VinoServer *
@@ -1902,6 +1929,19 @@ vino_server_set_lock_screen (VinoServer *server,
     }
 }
 
+static void
+vino_server_set_notify_on_connect (VinoServer *server,
+    gboolean notify_on_connect)
+{
+    g_return_if_fail (VINO_IS_SERVER (server));
+
+    if (server->priv->notify_on_connect == notify_on_connect)
+        return;
+
+    server->priv->notify_on_connect = notify_on_connect;
+    g_object_notify (G_OBJECT (server), "notify-on-connect");
+}
+
 gboolean
 vino_server_get_notify_on_connect (VinoServer *server)
 {
@@ -1918,3 +1958,23 @@ vino_server_get_status_icon (VinoServer *server)
   return server->priv->icon;
 }
 
+void
+vino_server_set_reject_incoming (VinoServer *server,
+    gboolean reject)
+{
+  g_return_if_fail (VINO_IS_SERVER (server));
+
+  if (server->priv->reject_incoming == reject)
+    return;
+
+  server->priv->reject_incoming = reject;
+  g_object_notify (G_OBJECT (server), "reject-incoming");
+}
+
+gboolean
+vino_server_get_reject_incoming (VinoServer *server)
+{
+  g_return_val_if_fail (VINO_IS_SERVER (server), FALSE);
+
+  return server->priv->reject_incoming;
+}
